@@ -38,10 +38,10 @@
 #include "config.h"
 
 #include "conf.h"
-#define DEBUG_FLAG DEBUG_CONF
+#define P11_DEBUG_FLAG P11_DEBUG_CONF
 #include "debug.h"
+#include "library.h"
 #include "private.h"
-#include "util.h"
 
 #include <sys/param.h>
 #include <sys/stat.h>
@@ -111,43 +111,6 @@ strequal (const char *one, const char *two)
 	return strcmp (one, two) == 0;
 }
 
-static char *
-strconcat (const char *first,
-           ...) GNUC_NULL_TERMINATED;
-
-static char *
-strconcat (const char *first,
-           ...)
-{
-	size_t length = 0;
-	const char *arg;
-	char *result, *at;
-	va_list va;
-
-	va_start (va, first);
-
-	for (arg = first; arg; arg = va_arg (va, const char*))
-		length += strlen (arg);
-
-	va_end (va);
-
-	at = result = malloc (length + 1);
-	return_val_if_fail (result != NULL, NULL);
-
-	va_start (va, first);
-
-	for (arg = first; arg; arg = va_arg (va, const char*)) {
-		length = strlen (arg);
-		memcpy (at, arg, length);
-		at += length;
-	}
-
-	va_end (va);
-
-	*at = 0;
-	return result;
-}
-
 /* -----------------------------------------------------------------------------
  * CONFIG PARSER
  */
@@ -167,13 +130,20 @@ read_config_file (const char* filename, int flags)
 		error = errno;
 		if ((flags & CONF_IGNORE_MISSING) &&
 		    (error == ENOENT || error == ENOTDIR)) {
-			_p11_debug ("config file does not exist");
+			p11_debug ("config file does not exist");
+			config = strdup ("\n");
+			return_val_if_fail (config != NULL, NULL);
+			return config;
+
+		} else if ((flags & CONF_IGNORE_ACCESS_DENIED) &&
+		           (error == EPERM || error == EACCES)) {
+			p11_debug ("config file is inaccessible");
 			config = strdup ("\n");
 			return_val_if_fail (config != NULL, NULL);
 			return config;
 		}
-		_p11_message ("couldn't open config file: %s: %s", filename,
-		              strerror (error));
+		p11_message ("couldn't open config file: %s: %s", filename,
+		             strerror (error));
 		errno = error;
 		return NULL;
 	}
@@ -183,14 +153,14 @@ read_config_file (const char* filename, int flags)
 	    (len = ftell (f)) == -1 ||
 	    fseek (f, 0, SEEK_SET) == -1) {
 		error = errno;
-		_p11_message ("couldn't seek config file: %s", filename);
+		p11_message ("couldn't seek config file: %s", filename);
 		errno = error;
 		return NULL;
 	}
 
 	config = malloc (len + 2);
 	if (config == NULL) {
-		_p11_message ("config file is too large to read into memory: %lu", len);
+		p11_message ("config file is too large to read into memory: %lu", len);
 		errno = ENOMEM;
 		return NULL;
 	}
@@ -198,7 +168,8 @@ read_config_file (const char* filename, int flags)
 	/* And read in one block */
 	if (fread (config, 1, len, f) != len) {
 		error = errno;
-		_p11_message ("couldn't read config file: %s", filename);
+		p11_message ("couldn't read config file: %s", filename);
+		free (config);
 		errno = error;
 		return NULL;
 	}
@@ -215,35 +186,36 @@ read_config_file (const char* filename, int flags)
 	return config;
 }
 
-int
-_p11_conf_merge_defaults (hashmap *map, hashmap *defaults)
+bool
+_p11_conf_merge_defaults (p11_dict *map,
+                          p11_dict *defaults)
 {
-	hashiter iter;
+	p11_dictiter iter;
 	void *key;
 	void *value;
 
-	_p11_hash_iterate (defaults, &iter);
-	while (_p11_hash_next (&iter, &key, &value)) {
+	p11_dict_iterate (defaults, &iter);
+	while (p11_dict_next (&iter, &key, &value)) {
 		/* Only override if not set */
-		if (_p11_hash_get (map, key))
+		if (p11_dict_get (map, key))
 			continue;
 		key = strdup (key);
-		return_val_if_fail (key != NULL, -1);
+		return_val_if_fail (key != NULL, false);
 		value = strdup (value);
-		return_val_if_fail (key != NULL, -1);
-		if (!_p11_hash_set (map, key, value))
-			return_val_if_reached (-1);
+		return_val_if_fail (key != NULL, false);
+		if (!p11_dict_set (map, key, value))
+			return_val_if_reached (false);
 	}
 
-	return 0;
+	return true;
 }
 
-hashmap *
+p11_dict *
 _p11_conf_parse_file (const char* filename, int flags)
 {
 	char *name;
 	char *value;
-	hashmap *map = NULL;
+	p11_dict *map = NULL;
 	char *data;
 	char *next;
 	char *end;
@@ -251,14 +223,14 @@ _p11_conf_parse_file (const char* filename, int flags)
 
 	assert (filename);
 
-	_p11_debug ("reading config file: %s", filename);
+	p11_debug ("reading config file: %s", filename);
 
 	/* Adds an extra newline to end of file */
 	data = read_config_file (filename, flags);
 	if (!data)
 		return NULL;
 
-	map = _p11_hash_create (_p11_hash_string_hash, _p11_hash_string_equal, free, free);
+	map = p11_dict_new (p11_dict_str_hash, p11_dict_str_equal, free, free);
 	return_val_if_fail (map != NULL, NULL);
 
 	next = data;
@@ -276,7 +248,7 @@ _p11_conf_parse_file (const char* filename, int flags)
 		/* Look for the break between name: value on the same line */
 		value = name + strcspn (name, ":");
 		if (!*value) {
-			_p11_message ("%s: invalid config line: %s", filename, name);
+			p11_message ("%s: invalid config line: %s", filename, name);
 			error = EINVAL;
 			break;
 		}
@@ -294,16 +266,16 @@ _p11_conf_parse_file (const char* filename, int flags)
 		value = strdup (value);
 		return_val_if_fail (value != NULL, NULL);
 
-		_p11_debug ("config value: %s: %s", name, value);
+		p11_debug ("config value: %s: %s", name, value);
 
-		if (!_p11_hash_set (map, name, value))
+		if (!p11_dict_set (map, name, value))
 			return_val_if_reached (NULL);
 	}
 
 	free (data);
 
 	if (error != 0) {
-		_p11_hash_free (map);
+		p11_dict_free (map);
 		map = NULL;
 		errno = error;
 	}
@@ -332,8 +304,8 @@ expand_user_path (const char *path)
 		pwd = getpwuid (getuid ());
 		if (!pwd) {
 			error = errno;
-			_p11_message ("couldn't lookup home directory for user %d: %s",
-			              getuid (), strerror (errno));
+			p11_message ("couldn't lookup home directory for user %d: %s",
+			             getuid (), strerror (errno));
 			errno = error;
 			return NULL;
 		}
@@ -355,12 +327,13 @@ expand_user_path (const char *path)
 }
 
 static int
-user_config_mode (hashmap *config, int defmode)
+user_config_mode (p11_dict *config,
+                  int defmode)
 {
 	const char *mode;
 
 	/* Whether we should use or override from user directory */
-	mode = _p11_hash_get (config, "user-config");
+	mode = p11_dict_get (config, "user-config");
 	if (mode == NULL) {
 		return defmode;
 	} else if (strequal (mode, "none")) {
@@ -372,20 +345,21 @@ user_config_mode (hashmap *config, int defmode)
 	} else if (strequal (mode, "override")) {
 		return CONF_USER_ONLY;
 	} else {
-		_p11_message ("invalid mode for 'user-config': %s", mode);
+		p11_message ("invalid mode for 'user-config': %s", mode);
 		return CONF_USER_INVALID;
 	}
 }
 
-hashmap *
+p11_dict *
 _p11_conf_load_globals (const char *system_conf, const char *user_conf,
                         int *user_mode)
 {
-	hashmap *config = NULL;
-	hashmap *uconfig = NULL;
-	hashmap *result = NULL;
+	p11_dict *config = NULL;
+	p11_dict *uconfig = NULL;
+	p11_dict *result = NULL;
 	char *path = NULL;
 	int error = 0;
+	int flags;
 	int mode;
 
 	/*
@@ -414,8 +388,9 @@ _p11_conf_load_globals (const char *system_conf, const char *user_conf,
 			goto finished;
 		}
 
-		/* Load up the user configuration */
-		uconfig = _p11_conf_parse_file (path, CONF_IGNORE_MISSING);
+		/* Load up the user configuration, ignore selinux denying us access */
+		flags = CONF_IGNORE_MISSING | CONF_IGNORE_ACCESS_DENIED;
+		uconfig = _p11_conf_parse_file (path, flags);
 		if (!uconfig) {
 			error = errno;
 			goto finished;
@@ -430,7 +405,7 @@ _p11_conf_load_globals (const char *system_conf, const char *user_conf,
 
 		/* If merging, then supplement user config with system values */
 		if (mode == CONF_USER_MERGE) {
-			if (_p11_conf_merge_defaults (uconfig, config) < 0) {
+			if (!_p11_conf_merge_defaults (uconfig, config)) {
 				error = errno;
 				goto finished;
 			}
@@ -438,7 +413,7 @@ _p11_conf_load_globals (const char *system_conf, const char *user_conf,
 
 		/* If user config valid at all, then replace system with what we have */
 		if (mode != CONF_USER_NONE) {
-			_p11_hash_free (config);
+			p11_dict_free (config);
 			config = uconfig;
 			uconfig = NULL;
 		}
@@ -452,8 +427,8 @@ _p11_conf_load_globals (const char *system_conf, const char *user_conf,
 
 finished:
 	free (path);
-	_p11_hash_free (config);
-	_p11_hash_free (uconfig);
+	p11_dict_free (config);
+	p11_dict_free (uconfig);
 	errno = error;
 	return result;
 }
@@ -498,11 +473,14 @@ calc_name_from_filename (const char *fname)
 	return name;
 }
 
-static int
-load_config_from_file (const char *configfile, const char *name, hashmap *configs)
+static bool
+load_config_from_file (const char *configfile,
+                       const char *name,
+                       p11_dict *configs,
+                       int flags)
 {
-	hashmap *config;
-	hashmap *prev;
+	p11_dict *config;
+	p11_dict *prev;
 	char *key;
 	int error = 0;
 
@@ -510,70 +488,78 @@ load_config_from_file (const char *configfile, const char *name, hashmap *config
 
 	key = calc_name_from_filename (name);
 	if (key == NULL) {
-		_p11_message ("invalid config filename, will be ignored in the future: %s", configfile);
+		p11_message ("invalid config filename, will be ignored in the future: %s", configfile);
 		key = strdup (name);
-		return_val_if_fail (key != NULL, -1);
+		return_val_if_fail (key != NULL, false);
 	}
 
-	config = _p11_conf_parse_file (configfile, 0);
+	config = _p11_conf_parse_file (configfile, flags);
 	if (!config) {
 		free (key);
-		return -1;
+		return false;
 	}
 
-	prev = _p11_hash_get (configs, key);
+	prev = p11_dict_get (configs, key);
 	if (prev == NULL) {
-		if (!_p11_hash_set (configs, key, config))
-			return_val_if_reached (-1);
+		if (!p11_dict_set (configs, key, config))
+			return_val_if_reached (false);
 		config = NULL;
 	} else {
-		if (_p11_conf_merge_defaults (prev, config) < 0)
+		if (!_p11_conf_merge_defaults (prev, config))
 			error = errno;
 		free (key);
 	}
 
 	/* If still set */
-	_p11_hash_free (config);
+	p11_dict_free (config);
 
 	if (error) {
 		errno = error;
-		return -1;
+		return false;
 	}
 
-	return 0;
+	return true;
 }
 
-static int
-load_configs_from_directory (const char *directory, hashmap *configs)
+static bool
+load_configs_from_directory (const char *directory,
+                             p11_dict *configs,
+                             int flags)
 {
 	struct dirent *dp;
 	struct stat st;
 	DIR *dir;
 	int error = 0;
-	int is_dir;
+	bool is_dir;
 	char *path;
 	int count = 0;
 
-	_p11_debug ("loading module configs in: %s", directory);
+	p11_debug ("loading module configs in: %s", directory);
 
 	/* First we load all the modules */
 	dir = opendir (directory);
 	if (!dir) {
 		error = errno;
-		if (errno == ENOENT || errno == ENOTDIR)
-			return 0;
-		_p11_message ("couldn't list directory: %s: %s", directory,
-		              strerror (error));
+		if ((flags & CONF_IGNORE_MISSING) &&
+		    (errno == ENOENT || errno == ENOTDIR)) {
+			p11_debug ("module configs do not exist");
+			return true;
+		} else if ((flags & CONF_IGNORE_ACCESS_DENIED) &&
+		           (errno == EPERM || errno == EACCES)) {
+			p11_debug ("couldn't list inacessible module configs");
+			return true;
+		}
+		p11_message ("couldn't list directory: %s: %s", directory,
+		             strerror (error));
 		errno = error;
-		return -1;
+		return false;
 	}
 
 	/* We're within a global mutex, so readdir is safe */
 	while ((dp = readdir(dir)) != NULL) {
 		path = strconcat (directory, "/", dp->d_name, NULL);
-		return_val_if_fail (path != NULL, -1);
+		return_val_if_fail (path != NULL, false);
 
-		is_dir = 0;
 #ifdef HAVE_STRUCT_DIRENT_D_TYPE
 		if(dp->d_type != DT_UNKNOWN) {
 			is_dir = (dp->d_type == DT_DIR);
@@ -582,14 +568,14 @@ load_configs_from_directory (const char *directory, hashmap *configs)
 		{
 			if (stat (path, &st) < 0) {
 				error = errno;
-				_p11_message ("couldn't stat path: %s", path);
+				p11_message ("couldn't stat path: %s", path);
 				free (path);
 				break;
 			}
 			is_dir = S_ISDIR (st.st_mode);
 		}
 
-		if (!is_dir && load_config_from_file (path, dp->d_name, configs) < 0) {
+		if (!is_dir && !load_config_from_file (path, dp->d_name, configs, flags)) {
 			error = errno;
 			free (path);
 			break;
@@ -603,33 +589,38 @@ load_configs_from_directory (const char *directory, hashmap *configs)
 
 	if (error) {
 		errno = error;
-		return -1;
+		return false;
 	}
 
-	return count;
+	return true;
 }
 
-hashmap *
-_p11_conf_load_modules (int mode, const char *system_dir, const char *user_dir)
+p11_dict *
+_p11_conf_load_modules (int mode,
+                        const char *package_dir,
+                        const char *system_dir,
+                        const char *user_dir)
 {
-	hashmap *configs;
+	p11_dict *configs;
 	char *path;
 	int error = 0;
+	int flags;
 
 	/* A hash table of name -> config */
-	configs = _p11_hash_create (_p11_hash_string_hash, _p11_hash_string_equal,
-	                            free, (hash_destroy_func)_p11_hash_free);
+	configs = p11_dict_new (p11_dict_str_hash, p11_dict_str_equal,
+	                        free, (p11_destroyer)p11_dict_free);
 
 	/* Load each user config first, if user config is allowed */
 	if (mode != CONF_USER_NONE) {
+		flags = CONF_IGNORE_MISSING | CONF_IGNORE_ACCESS_DENIED;
 		path = expand_user_path (user_dir);
 		if (!path)
 			error = errno;
-		else if (load_configs_from_directory (path, configs) < 0)
+		else if (!load_configs_from_directory (path, configs, flags))
 			error = errno;
 		free (path);
 		if (error != 0) {
-			_p11_hash_free (configs);
+			p11_dict_free (configs);
 			errno = error;
 			return NULL;
 		}
@@ -641,9 +632,11 @@ _p11_conf_load_modules (int mode, const char *system_dir, const char *user_dir)
 	 * loaded above (in the user configs) then they're loaded here.
 	 */
 	if (mode != CONF_USER_ONLY) {
-		if (load_configs_from_directory (system_dir, configs) < 0) {
+		flags = CONF_IGNORE_MISSING;
+		if (!load_configs_from_directory (system_dir, configs, flags) ||
+		    !load_configs_from_directory (package_dir, configs, flags)) {
 			error = errno;
-			_p11_hash_free (configs);
+			p11_dict_free (configs);
 			errno = error;
 			return NULL;
 		}
@@ -652,20 +645,20 @@ _p11_conf_load_modules (int mode, const char *system_dir, const char *user_dir)
 	return configs;
 }
 
-int
+bool
 _p11_conf_parse_boolean (const char *string,
-                         int default_value)
+                         bool default_value)
 {
 	if (!string)
 		return default_value;
 
 	if (strcmp (string, "yes") == 0) {
-		return 1;
+		return true;
 	} else if (strcmp (string, "no") == 0) {
-		return 0;
+		return false;
 	} else {
-		_p11_message ("invalid setting '%s' defaulting to '%s'",
-		              string, default_value ? "yes" : "no");
+		p11_message ("invalid setting '%s' defaulting to '%s'",
+		             string, default_value ? "yes" : "no");
 		return default_value;
 	}
 }
